/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.extensions

import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import net.moonleay.bedge.data.database.repository.UserRepository
import net.moonleay.bedge.jobs.component.JobManager
import net.moonleay.bedge.util.EmbedColor
import net.moonleay.bedge.util.Logger
import net.moonleay.bedge.util.MessageUtil
import net.moonleay.bedge.util.TimeUtil


class AwakeExtension : Extension() {
    override val name = "awake"
    override suspend fun setup() {
        publicSlashCommand {
            name = "awake"
            description = "Set yourself as awake"
            this.action {
                val u = this.user.asUser()
                Logger.out("User ${u.username} wants to be awake")
                if (UserRepository.doesUserExist(u.id.value)){
                    val ud = UserRepository.getUserByID(u.id.value)
                    if (ud.isAwake) {
                        // User is already awake

                        if (ud.currentStreak == 0) {
                            // User failed last time
                            this.respond {
                                this.embeds.add(
                                    MessageUtil.getEmbed(
                                        EmbedColor.ERROR,
                                        "You failed last time!!",
                                        "You failed to wake up at ${TimeUtil.getHourAndMinuteFromStamp(ud.nextWakeup)}!\n" +
                                                "You can try again tomorrow with `/time`.",
                                        u.username
                                    )
                                )
                            }
                            return@action
                        }
                        this.respond {
                            this.embeds.add(
                                MessageUtil.getEmbed(
                                    EmbedColor.ERROR,
                                    "You are already awake!",
                                    "You can set a wakeup time for tomorrow with `/time`",
                                    u.username,
                                )
                            )
                        }
                        return@action
                    }
                    // set awake
                    ud.isAwake = true
                    ud.lastWakeup = System.currentTimeMillis()

                    // update the streak
                    ud.currentStreak++
                    if (ud.currentStreak > ud.longestStreak) {
                        ud.longestStreak = ud.currentStreak
                    }

                    // hand out coins for streak & success
                    var streakCoins = ud.currentStreak / 7
                    if (streakCoins > 5)
                        streakCoins = 5
                    ud.coins += (1 + streakCoins)
                    ud.coinsCollected += (1 + streakCoins)

                    // grant xp and check for lvlup
                    val neededXpForNextLvl = (10 + ((100*0.1) * (ud.level - 1))).toInt()
                    val bonusXp = (1.5 * ud.currentStreak).toInt()
                    ud.xp += 10 + bonusXp
                    if (ud.xp >= neededXpForNextLvl) {
                        ud.xp -= neededXpForNextLvl
                        ++ud.level
                    }

                    // update user
                    UserRepository.update(ud)

                    // kill job
                    JobManager.killNamedJob("WakeupJob${u.id.value.toLong()}")

                    // respond to user
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.SUCCESS,
                                "Good morning, ${u.username}!",
                                ud.customWakeupMessage.replace("#user", u.mention) + "\n" +
                                        "\n" +
                                        "${if (ud.currentStreak > 0) "[${ud.currentStreak} day streak]" else ""}\n" +
                                        "${ud.coins} coin${if (ud.coins >= 1) "s" else ""} (+${streakCoins+1})\n" +
                                        "lvl ${ud.level} (${ud.xp}/${neededXpForNextLvl} xp) [+${10 + bonusXp} xp]\n",
                                u.username,
                            )
                        )
                    }
                } else {
                    // User is running for the first time
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "You are not registered!",
                                "Please register with `/time`",
                                u.username,
                            )
                        )
                    }
                }
            }
        }
    }
}
