/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.commands.converters.impl.string
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import net.moonleay.bedge.data.database.entry.UserData
import net.moonleay.bedge.data.database.repository.UserRepository
import net.moonleay.bedge.jobs.WakeupJob
import net.moonleay.bedge.jobs.component.JobManager
import net.moonleay.bedge.util.*

class TimeExtension : Extension() {

    override val name = "time"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        publicSlashCommand(::TimeArguments) {
            name = "time"
            description = "Set the wakeup time for tomorrow"

            this.action {
                val u = this.user.asUser()
                val targetTime = this.arguments.time
                Logger.out("User ${u.username} wants to be awake at $targetTime")

                val timeToWake = TimeUtil.getZdtFromTime(targetTime, "UTC")
                val cronJobString = TimeUtil.getCronjobStringFromDate(timeToWake)
                val targetChannelId = this.channel.asChannel().id.value.toLong()
                lateinit var ud: UserData

                if (UserRepository.doesUserExist(u.id.value)) {
                    // Update existing user
                    ud = UserRepository.getUserByID(u.id.value)
                    ud.nextWakeup = timeToWake.toEpochSecond() * 1000 // ms
                    ud.nextWakeupCron = cronJobString
                    ud.isAwake = false
                    UserRepository.update(ud)
                } else {
                    // create new user
                    ud = UserData (
                        -1,
                        u.id.value.toLong(),
                        System.currentTimeMillis(), // account creation date
                        0,
                        0,
                        false,
                        0,
                        0,
                        0,
                        1,
                        0,
                        targetChannelId,
                        "Don't forget to go to bed, #user!",
                        8, // hrs
                        "#user is now awake!",
                        timeToWake.toEpochSecond() * 1000, // ms
                        cronJobString,
                        0,
                        false,
                        "UTC"
                    )
                    UserRepository.write(ud)
                }
                JobManager.killNamedJob("WakeupJob${u.id.value.toLong()}") // Make sure that there is no job, which is already running
                JobManager.addJob(WakeupJob("WakeupJob${u.id.value.toLong()}", cronJobString, u.id.value.toLong()))

                Logger.out("User ${u.username} is now set to be awake at $targetTime")
                this.respond {
                    this.embeds.add(MessageUtil.getEmbed(
                        EmbedColor.SUCCESS,
                        "Wakeup time set",
                        "${u.mention} is now set to be awake at $targetTime.\n" +
                                "Have a restfull sleep!\n" +
                                "***You can change your wakeup time by running /time again!***",
                        u.username
                    ))
                }
            }
        }
    }

    inner class TimeArguments : Arguments() {

        val time by string {
            this.name = "time"
            this.description = "The time you want to be awake at. Format: \"HH:mm\""
        }
    }
}
