/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.application.slash.converters.impl.enumChoice
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import dev.kord.common.entity.Snowflake
import dev.kord.core.entity.Guild
import kotlinx.coroutines.flow.map
import net.moonleay.bedge.data.database.entry.UserData
import net.moonleay.bedge.data.database.repository.UserRepository
import net.moonleay.bedge.extensions.component.ListTypes
import net.moonleay.bedge.util.EmbedColor
import net.moonleay.bedge.util.MessageUtil

class TopExtension : Extension() {

    override val name = "top"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        publicSlashCommand(::TopArguments) {
            name = "top"
            description = "See the top lists"

            this.action {
                val u = this.user.asUser()
                val g = this.guild!!.asGuild()
                val targetList = this.arguments.listType
                val all = UserRepository.getAllUsers()
                val allInGuild = all.filter { g.getMemberOrNull(Snowflake(it.userid)) != null }
                when(targetList) {
                    ListTypes.TOPSTREAK -> {
                        allInGuild.sortedByDescending { it.longestStreak }
                    }
                    ListTypes.STREAK -> {
                        allInGuild.sortedByDescending { it.currentStreak }
                    }
                    ListTypes.TOPCOINS -> {
                        allInGuild.sortedByDescending { it.coinsCollected }
                    }
                    ListTypes.COINS -> {
                        allInGuild.sortedByDescending { it.coins }
                    }
                    ListTypes.FAILS -> {
                        allInGuild.sortedByDescending { it.numberOfFails }
                    }
                }
                var msg = ""
                for (i in 0..if (allInGuild.size > 10) 9 else allInGuild.size-1) {
                    val user = allInGuild[i]
                    val row = getRow(user, g, targetList)
                    msg += "${i+1}. ${row[0]}: ${row[1]}\n"
                }
                this.respond {
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            EmbedColor.INFO,
                            targetList.readableName,
                            msg,
                            u.username,
                        )
                    )
                }
            }
        }
    }


    private suspend fun getRow(user: UserData, g: Guild, type: ListTypes): Array<String> {
        val typeResult: String = when(type) {
            ListTypes.TOPSTREAK -> user.longestStreak.toString()
            ListTypes.STREAK -> user.currentStreak.toString()
            ListTypes.TOPCOINS -> user.coinsCollected.toString()
            ListTypes.COINS -> user.coins.toString()
            ListTypes.FAILS -> user.numberOfFails.toString()
        }
        return arrayOf(g.getMember(Snowflake(user.userid)).mention, typeResult)
    }

    inner class TopArguments : Arguments() {

        val listType by enumChoice<ListTypes> {
            this.name = "list"
            this.typeName = "list"
            this.description = "The list you want to view"
        }
    }
}
