/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.features

import net.moonleay.bedge.data.database.repository.UserRepository
import net.moonleay.bedge.jobs.WakeupJob
import net.moonleay.bedge.jobs.component.JobManager
import net.moonleay.bedge.util.Logger

object WakeupFeature {

    fun createWakeupCronjobs() {
        val users = UserRepository.getAllUsers()
        users.forEach { user ->
            if (user.nextWakeup > System.currentTimeMillis() && !user.isAwake) {
                JobManager.addJob(WakeupJob("WakeupJob${user.id}", user.nextWakeupCron, user.userid))
                Logger.out("Created wakeup job for user ${user.id}")
            } else if (user.nextWakeup < System.currentTimeMillis() && !user.isAwake) {
                // Set the user awake since the wakeup time was during downtime
                user.isAwake = true
                // don't increase the streak, since it is not certain that the user was awake
                // user.currentStreak++
                user.lastWakeup = System.currentTimeMillis()
                Logger.out("Updated user ${user.id} to be awake")
            }
        }
        Logger.out("Finished updating wakeup jobs")
    }
}
