/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.jobs

import dev.inmo.krontab.KronScheduler
import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.channel.createEmbed
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.entity.channel.MessageChannel
import kotlinx.coroutines.Job
import net.moonleay.bedge.Bot
import net.moonleay.bedge.data.database.repository.UserRepository
import net.moonleay.bedge.jobs.component.CronjobType
import net.moonleay.bedge.jobs.component.ICronjob
import net.moonleay.bedge.jobs.component.JobManager
import net.moonleay.bedge.util.EmbedColor
import net.moonleay.bedge.util.Logger
import net.moonleay.bedge.util.MessageUtil
import net.moonleay.bedge.util.TimeUtil

class WakeupJob(override val jobName: String, override val jobIncoming: String, val userId: Long) : ICronjob {
    override val jobType: CronjobType
        get() = CronjobType.ONCE
    override val continueJob: Boolean
        get() = false
    override lateinit var cronjobJob: Job
    override lateinit var scheduler: KronScheduler

    override suspend fun jobFunction() {
        Logger.out("WakeupJob, running \"$jobName\"")
        val user = UserRepository.getUserByID(userId)
        if (!user.isAwake){
            // Failed
            val userasuser = Bot.bot.kordRef.getUser(Snowflake(userId))!!.asUser()
            val brokenStreak = user.currentStreak
            val isBiggestStreakYet = user.currentStreak > user.longestStreak
            ++user.numberOfFails
            user.currentStreak = 0
            user.isAwake = true
            user.lastWakeup = System.currentTimeMillis()
            UserRepository.update(user)
            if(Bot.bot.kordRef.getChannel(Snowflake(user.targetChannelId)) != null){
                val ch = Bot.bot.kordRef.getChannelOf<MessageChannel>(Snowflake(user.targetChannelId))
                ch!!.createMessage {
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            EmbedColor.ERROR,
                            "You failed to wake up!",
                            "${userasuser.mention} failed to wake up at ${TimeUtil.getHourAndMinuteFromStamp(user.nextWakeup)}!\n" +
                                    "You lost a streak of $brokenStreak" +
                                    if(isBiggestStreakYet) ", which was your biggest streak yet." else "." +
                                    "\nYou can try again tomorrow with `/time`",
                            "Automated Message",
                        )
                    )
                }
            }
        }
        // else: Success

        JobManager.killJob(this)
    }
}
